from PyQt4 import QtCore
import PyQt4.QtGui as QtWidgets


class RegressionParameters(QtWidgets.QDialog):
	
	def __init__(self, parent):
		super(RegressionParameters, self).__init__(parent)
		
		self.__parent = parent
		self.__currentTab = 0
		
		# Linear Regression tab
		linRegLossFunctionLabel = QtWidgets.QLabel('Loss function')
		linRegLossFunctionParameterLabel = QtWidgets.QLabel('Parameter of the loss function')
		self.linRegLossFunctionCombobox = QtWidgets.QComboBox()
		self.linRegLossFunctionCombobox.addItem('l2 norm')
		self.linRegLossFunctionCombobox.addItem('Huber loss')
		self.linRegLossFunctionCombobox.currentIndexChanged.connect(self.onLinRegLossFunctionChanged)
		self.linRegLossFunctionParameterEdit = QtWidgets.QLineEdit('0.001')
		self.linRegLossFunctionParameterEdit.setEnabled(False)
		
		linRegGrid = QtWidgets.QGridLayout();
		linRegGrid.setSpacing(10)
		linRegGrid.addWidget(linRegLossFunctionLabel, 0, 0)
		linRegGrid.addWidget(self.linRegLossFunctionCombobox, 0, 1)
		linRegGrid.addWidget(linRegLossFunctionParameterLabel, 1, 0)
		linRegGrid.addWidget(self.linRegLossFunctionParameterEdit, 1, 1)
		
		linRegFrame = QtWidgets.QGroupBox()
		linRegFrame.setTitle('Linear Regression')		
		linRegFrame.setLayout(linRegGrid)
		
		linRegLayout = QtWidgets.QVBoxLayout()
		linRegLayout.addWidget(linRegFrame)
		linRegLayout.addStretch(1)
		
		self.linRegTab = QtWidgets.QWidget()
		self.linRegTab.setLayout(linRegLayout)
		
		# SVR properties
		SVRAlgorithmLabel = QtWidgets.QLabel('Algorithm')
		SVRKernelLabel = QtWidgets.QLabel('Kernel')
		SVRCLabel = QtWidgets.QLabel('C')
		SVREpsilonLabel = QtWidgets.QLabel('Epsilon')
		SVRGammaLabel = QtWidgets.QLabel('Gamma')
		SVRCoef0Label = QtWidgets.QLabel('Coef0')
		SVRDegreeLabel = QtWidgets.QLabel('Degree')
		self.SVRAlgorithmCombobox = QtWidgets.QComboBox()
		self.SVRAlgorithmCombobox.addItem('SVR (scikit-learn)')
		
		self.SVRKernelCombobox = QtWidgets.QComboBox()
		self.SVRKernelCombobox.addItem('linear')
		self.SVRKernelCombobox.addItem('polynomial')
		self.SVRKernelCombobox.addItem('radial basis functions')
		self.SVRKernelCombobox.addItem('sigmoid')
		self.SVRKernelCombobox.currentIndexChanged.connect(self.onSVRKernelChanged)
		self.SVRCEdit = QtWidgets.QLineEdit('1.0')
		self.SVRCEdit.setToolTip('Penalty parameter of the error term')
		self.SVREpsilonEdit = QtWidgets.QLineEdit('0.1')
		self.SVREpsilonEdit.setToolTip('Specifies the epsilon-tube within which no penalty is associated in the training loss function with points predicted within a distance epsilon from the actual value.')
		self.SVRGammaEdit = QtWidgets.QLineEdit('0.5')
		self.SVRGammaEdit.setToolTip('Kernel coefficient')
		self.SVRCoef0Edit = QtWidgets.QLineEdit('0.0')
		self.SVRCoef0Edit.setToolTip('Independent term in kernel function')
		self.SVRDegreeEdit = QtWidgets.QLineEdit('3')
		self.SVRDegreeEdit.setToolTip('Degree of the polynomial kernel function')
		self.onSVRKernelChanged(0)
		
		SVRGrid = QtWidgets.QGridLayout();
		SVRGrid.setSpacing(10)
		SVRGrid.addWidget(SVRAlgorithmLabel, 0, 0)
		SVRGrid.addWidget(self.SVRAlgorithmCombobox, 0, 1)
		SVRGrid.addWidget(SVRKernelLabel, 1, 0)
		SVRGrid.addWidget(self.SVRKernelCombobox, 1, 1)
		SVRGrid.addWidget(SVRCLabel, 2, 0)
		SVRGrid.addWidget(self.SVRCEdit, 2, 1)
		SVRGrid.addWidget(SVREpsilonLabel, 3, 0)
		SVRGrid.addWidget(self.SVREpsilonEdit, 3, 1)
		SVRGrid.addWidget(SVRGammaLabel, 4, 0)
		SVRGrid.addWidget(self.SVRGammaEdit, 4, 1)
		SVRGrid.addWidget(SVRCoef0Label, 5, 0)
		SVRGrid.addWidget(self.SVRCoef0Edit, 5, 1)
		SVRGrid.addWidget(SVRDegreeLabel, 6, 0)
		SVRGrid.addWidget(self.SVRDegreeEdit, 6, 1)
		
		SVRFrame = QtWidgets.QGroupBox()
		SVRFrame.setTitle('Support Vector Regression')
		SVRFrame.setLayout(SVRGrid)
		
		SVRLayout = QtWidgets.QVBoxLayout()
		SVRLayout.addWidget(SVRFrame)
		SVRLayout.addStretch(1)
		
		self.SVRTab = QtWidgets.QWidget()
		self.SVRTab.setLayout(SVRLayout)
		
		# Regression Tree
		RegressionForestNumTreesLabel = QtWidgets.QLabel('Num. Trees')
		RegressionForestDepthLabel = QtWidgets.QLabel('Depth')
		RegressionForestNumSplitLabel = QtWidgets.QLabel('Random Splits')
		self.RegressionForestNumTreesEdit = QtWidgets.QLineEdit('1')
		self.RegressionForestDepthEdit = QtWidgets.QLineEdit('1')
		self.RegressionForestNumSplitEdit = QtWidgets.QLineEdit('4')
		
		RegressionForestGrid = QtWidgets.QGridLayout();
		RegressionForestGrid.setSpacing(10)
		RegressionForestGrid.addWidget(RegressionForestNumTreesLabel, 0, 0)
		RegressionForestGrid.addWidget(self.RegressionForestNumTreesEdit, 0, 1)
		RegressionForestGrid.addWidget(RegressionForestDepthLabel, 1, 0)
		RegressionForestGrid.addWidget(self.RegressionForestDepthEdit, 1, 1)
		RegressionForestGrid.addWidget(RegressionForestNumSplitLabel, 2, 0)
		RegressionForestGrid.addWidget(self.RegressionForestNumSplitEdit, 2, 1)
		
		RegressionForestFrame = QtWidgets.QGroupBox()
		RegressionForestFrame.setTitle('Regression Tree')		
		RegressionForestFrame.setLayout(RegressionForestGrid)
		
		RegressionForestLayout = QtWidgets.QVBoxLayout()
		RegressionForestLayout.addWidget(RegressionForestFrame)
		RegressionForestLayout.addStretch(1)
		
		self.RegressionForestTab = QtWidgets.QWidget()
		self.RegressionForestTab.setLayout(RegressionForestLayout)
		
		
		# Create notebook tabs		
		self.notebook = QtWidgets.QTabWidget()
		self.notebook.addTab(self.linRegTab, "LinReg")
		self.notebook.addTab(self.SVRTab, "SVR")
		self.notebook.addTab(self.RegressionForestTab, "Forest")
		self.notebook.currentChanged.connect(self.onTabChanged)
		
		
		# OK and Cancel buttons
		self.buttons = QtWidgets.QDialogButtonBox(
			QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
			QtCore.Qt.Horizontal, self)
		self.buttons.button(QtWidgets.QDialogButtonBox.Cancel).setText("Cancel")
		self.buttons.button(QtWidgets.QDialogButtonBox.Ok).setText("Compute regression")

		self.buttons.accepted.connect(self.onAccept)
		self.buttons.rejected.connect(self.onReject)
		

		layout = QtWidgets.QVBoxLayout(self)
		layout.addWidget(self.notebook)		
		layout.addWidget(self.buttons)
		
		self.setWindowTitle('Regression parameters')
		self.reject = self.onReject


	def errorMsg(self, msg):
		QtWidgets.QMessageBox.warning(self, 'Error', msg, QtWidgets.QMessageBox.Ok, QtWidgets.QMessageBox.Ok)


	def testParameter(self, func, msg):
		try:
			func()
		except:
			self.errorMsg(msg)
			return False
		return True


	def checkParameters(self, tab):
		if tab == 0: # Linear Regression parameters
			if not self.testParameter(self.getLinRegLossFunctionParam, 'The parameter of the loss function has to be a floating point number!'):
				return False
			
			a = self.getLinRegLossFunctionParam()
			if a <= 0: 
				self.errorMsg('The parameter of the loss function has to be positive!')
				return False
		elif tab == 1: # SVR parameters
			if not self.testParameter(self.getSVRC, 'The penalty parameter C has to be a floating point number!'):
				return False
		
			if not self.testParameter(self.getSVREpsilon, 'The Epsilon parameter has to be a floating point number!'):
				return False
		
			if not self.testParameter(self.getSVRGamma, 'The kernel parameter Gamma has to be a floating point number!'):
				return False

			if not self.testParameter(self.getSVRCoef0, 'The independent term in the kernel function Coef0 has to be a floating point number!'):
				return False
		
			if not self.testParameter(self.getSVRDegree, 'The degree of the polynomial kernel function has to be an integer number!'):
				return False
		
			degree = self.getSVRDegree()
			if degree < 1:
				self.errorMsg('The degree of the polynomial kernel has to be at least 1!')
				return False
			
			C = self.getSVRC()
			if C <= 0:
				self.errorMsg('The penalty parameter C has to be positive!')
				return False
			
			eps = self.getSVREpsilon()
			if eps <= 0:
				self.errorMsg('The Epsilon parameter has to be positive!')
				return False
			
			gamma = self.getSVRGamma()
			if gamma <= 0:
				self.errorMsg('The kernel parameter Gamma has to be positive!')
				return False
		elif tab == 3: # Regression Forest
			if not self.testParameter(self.getRegressionForestNumTrees, 'The number of trees has to be an integer number > 0!'):
				return False
			if not self.testParameter(self.getRegressionForestDepth, 'The tree depth has to be an integer number > 0!'):
				return False
			if not self.testParameter(self.getRegressionForestNumSplit, 'The number of random split functions to be an integer number > 0!'):
				return False
		return True


	def onAccept(self):
		if not self.checkParameters(self.notebook.currentIndex()):
			return		
		super(RegressionParameters, self).accept()


	def onReject(self):
		self.restoreParameters()
		super(RegressionParameters, self).reject()


	def showEvent(self, event):
		self.__parameters = self.getParameters()


	def getParameters(self):
		params = {}
		params['LinReg_loss_function'] = self.linRegLossFunctionCombobox.currentIndex()
		params['LinReg_loss_function_param'] = self.linRegLossFunctionParameterEdit.text()
		params['svr_algo'] = self.SVRAlgorithmCombobox.currentIndex()
		params['svr_kernel'] = self.SVRKernelCombobox.currentIndex()
		params['svr_c'] = self.SVRCEdit.text()
		params['svr_epsilon'] = self.SVREpsilonEdit.text()
		params['svr_gamma'] = self.SVRGammaEdit.text()
		params['svr_coef0'] = self.SVRCoef0Edit.text()
		params['svr_degree'] = self.SVRDegreeEdit.text()
		params['regforest_num_trees'] = self.RegressionForestNumTreesEdit.text()
		params['regforest_depth'] = self.RegressionForestDepthEdit.text()
		params['regforest_num_split'] = self.RegressionForestNumSplitEdit.text()
		return params

		
	def restoreParameters(self):
		params = self.__parameters
		self.linRegLossFunctionCombobox.setCurrentIndex(params['LinReg_loss_function'])
		self.linRegLossFunctionParameterEdit.setText(params['LinReg_loss_function_param'])
		self.SVRAlgorithmCombobox.setCurrentIndex(params['svr_algo'])
		self.SVRKernelCombobox.setCurrentIndex(params['svr_kernel'])
		self.SVRCEdit.setText(params['svr_c'])
		self.SVREpsilonEdit.setText(params['svr_epsilon'])
		self.SVRGammaEdit.setText(params['svr_gamma'])
		self.SVRCoef0Edit.setText(params['svr_coef0'])
		self.SVRDegreeEdit.setText(params['svr_degree'])
		self.RegressionForestNumTreesEdit.setText(params['regforest_num_trees'])
		self.RegressionForestDepthEdit.setText(params['regforest_depth'])
		self.RegressionForestNumSplitEdit.setText(params['regforest_num_split'])


	def onTabChanged(self, idx):
		if not self.checkParameters(self.__currentTab):
			currentTab = self.__currentTab
			self.__currentTab = idx
			self.notebook.setCurrentIndex(currentTab)
		else:
			self.__currentTab = idx


	def onLinRegLossFunctionChanged(self, idx):
		if idx == 0: # l2 norm
			self.linRegLossFunctionParameterEdit.setEnabled(False)
		else: # Huber loss
			self.linRegLossFunctionParameterEdit.setEnabled(True)


	def onSVRKernelChanged(self, idx):
		if idx == 0: # linear
			self.SVRGammaEdit.setEnabled(False)
			self.SVRCoef0Edit.setEnabled(False)
			self.SVRDegreeEdit.setEnabled(False)
		elif idx == 1: # polynomial
			self.SVRGammaEdit.setEnabled(True)
			self.SVRCoef0Edit.setEnabled(True)
			self.SVRDegreeEdit.setEnabled(True)
		elif idx == 2: # rbf
			self.SVRGammaEdit.setEnabled(True)
			self.SVRCoef0Edit.setEnabled(False)
			self.SVRDegreeEdit.setEnabled(False)
		else: # sigmoid
			self.SVRGammaEdit.setEnabled(True)
			self.SVRCoef0Edit.setEnabled(True)
			self.SVRDegreeEdit.setEnabled(False)


	def setTab(self, tab):
		if tab >= 0:
			self.buttons.button(QtWidgets.QDialogButtonBox.Ok).setText("Compute regression")
			self.notebook.setCurrentIndex(tab - 1)
			self.__currentTab = tab - 1
			for i in range(self.notebook.count()):
				if i == tab - 1:
					self.notebook.setTabEnabled(i, True)
				else:
					self.notebook.setTabEnabled(i, False)
		else:
			self.buttons.button(QtWidgets.QDialogButtonBox.Ok).setText("Ok")
			self.notebook.setCurrentIndex(0)
			self.__currentTab = 0
			for i in range(self.notebook.count()):
				self.notebook.setTabEnabled(i, True)


	def getLinRegLossFunction(self):
		idx = self.linRegLossFunctionCombobox.currentIndex()
		if idx == 0:
			return 'l2'
		elif idx == 1:
			return 'huber'
		else:
			return None


	def getLinRegLossFunctionParam(self):
		return float(self.linRegLossFunctionParameterEdit.text())


	def getSVRAlgorithm(self):
		idx = self.SVRAlgorithmCombobox.currentIndex()
		if idx == 0:
			return 'SVR'
		else:
			return None


	def getSVRKernel(self):
		kernel = self.SVRKernelCombobox.currentIndex()
		if kernel == 0:
			return 'linear'
		elif kernel == 1:
			return 'poly'
		elif kernel == 2:
			return 'rbf'
		else:
			return 'sigmoid'


	def getSVRC(self):
		return float(self.SVRCEdit.text())


	def getSVREpsilon(self):
		return float(self.SVREpsilonEdit.text())


	def getSVRGamma(self):
		return float(self.SVRGammaEdit.text())


	def getSVRCoef0(self):
		return float(self.SVRCoef0Edit.text())


	def getSVRDegree(self):
		return int(self.SVRDegreeEdit.text())

	def getRegressionForestNumTrees(self):
		return int(self.RegressionForestNumTreesEdit.text())
	
	def getRegressionForestDepth(self):
		return int(self.RegressionForestDepthEdit.text())
	
	def getRegressionForestNumSplit(self):
		return int(self.RegressionForestNumSplitEdit.text())

